@@ -16,9 +16,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
    As a special exception, the authors of SANE give permission for
    additional uses of the libraries contained in this release of SANE.
@@ -924,9 +922,9 @@ sane_open (SANE_String_Const name, SANE_Handle * h)
 	  o.cap = SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT;
 	  break;
 	case CS2_OPTION_FOCUS:
-	  o.name = "focus";
-	  o.title = "Focus position";
-	  o.desc = "Focus position for manual focus";
+	  o.name = SANE_NAME_FOCUS;
+	  o.title = SANE_TITLE_FOCUS;
+	  o.desc = SANE_DESC_FOCUS;
 	  o.type = SANE_TYPE_INT;
 	  o.unit = SANE_UNIT_NONE;
 	  o.size = WSIZE;
@@ -944,9 +942,9 @@ sane_open (SANE_String_Const name, SANE_Handle * h)
 	    }
 	  break;
 	case CS2_OPTION_AUTOFOCUS:
-	  o.name = "autofocus";
-	  o.title = "Autofocus now";
-	  o.desc = "Autofocus now";
+	  o.name = SANE_NAME_AUTOFOCUS;
+	  o.title = SANE_TITLE_AUTOFOCUS;
+	  o.desc = SANE_DESC_AUTOFOCUS;
 	  o.type = SANE_TYPE_BUTTON;
 	  o.cap = SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT;
 	  break;
