@@ -16,9 +16,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
    As a special exception, the authors of SANE give permission for
    additional uses of the libraries contained in this release of SANE.
@@ -567,7 +565,7 @@ static void gl124_init_motor_regs_scan(Genesys_Device* dev,
     reg->set16(REG_FSHDEC, fast_table.table.size());
     reg->set16(REG_FMOVNO, fast_table.table.size());
 
-  /* substract acceleration distance from feedl */
+  /* subtract acceleration distance from feedl */
   feedl=feed_steps;
     feedl <<= static_cast<unsigned>(motor_profile.step_type);
 
@@ -1058,7 +1056,7 @@ void CommandSetGl124::send_shading_data(Genesys_Device* dev, const Genesys_Senso
   for(i=0;i<3;i++)
     {
       /* copy data to work buffer and process it */
-          /* coefficent destination */
+          /* coefficient destination */
       ptr = buffer.data();
 
       /* iterate on both sensor segment */
