@@ -16,9 +16,7 @@
     General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-    MA 02111-1307, USA.
+    along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
     As a special exception, the authors of SANE give permission for
     additional uses of the libraries contained in this release of SANE.
@@ -362,7 +360,7 @@ static void gl842_init_motor_regs_scan(Genesys_Device* dev,
         reg->set8(REG_0x80, vref);
     }
 
-    // substract acceleration distance from feedl
+    // subtract acceleration distance from feedl
     unsigned feedl = feed_steps;
     feedl <<= static_cast<unsigned>(motor_profile.step_type);
 
