@@ -15,9 +15,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
    As a special exception, the authors of SANE give permission for
    additional uses of the libraries contained in this release of SANE.
@@ -181,7 +179,7 @@ static int devlist_count;	/* Number of element in the list */
  */
 static SANE_Device **devlist = NULL;
 
-/* round() is c99, so we provide our own, though this version wont return -0 */
+/* round() is c99, so we provide our own, though this version won't return -0 */
 static double
 round2(double x)
 {
