@@ -16,9 +16,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
    As a special exception, the authors of SANE give permission for
    additional uses of the libraries contained in this release of SANE.
@@ -2251,7 +2249,7 @@ sane_read (SANE_Handle handle, SANE_Byte * data, SANE_Int max_length,
   *length = nread;
   /* Check whether we are scanning with a depth of 16 bits/pixel and whether
      server and client have different byte order. If this is true, then it's
-     neccessary to check whether read returned an odd number. If an odd number
+     necessary to check whether read returned an odd number. If an odd number
      has been returned, we must save the last byte.
   */
   if ((depth == 16) && (server_big_endian != client_big_endian))
