@@ -53,9 +53,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
- * MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * As a special exception, the authors of SANE give permission for
  * additional uses of the libraries contained in this release of SANE.
@@ -81,7 +79,7 @@
  */
 #include "plustek-pp_scan.h"
 
-/*************************** some definitons *********************************/
+/*************************** some definitions *********************************/
 
 /* #define _A3I_EN */
 
@@ -203,7 +201,7 @@ static void motorP96GetStartStopGap( pScanData ps, Bool fCheckState )
 
 
 /*.............................................................................
- * wait for the ScanState stop or ScanState reachs the dwScanStateCount
+ * wait for the ScanState stop or ScanState reaches the dwScanStateCount
  */
 static Bool motorCheckMotorPresetLength( pScanData ps )
 {
@@ -1912,6 +1910,7 @@ static void motorP96SetupRunTable( pScanData ps )
 			    case 3:
 					if (*(p.pb + 2))
 					    bColor = 1;
+                                        // fall through
 			    case 2:
 					if (*(p.pb + 1))
 			    		bColor++;
@@ -2039,7 +2038,7 @@ static void motorGoHalfStep1( pScanData ps )
 }
 
 /*.............................................................................
- * when loosing data, we use this function to go back some lines and read them
+ * when losing data, we use this function to go back some lines and read them
  * again...
  */
 static void motorP96WaitBack( pScanData ps )
@@ -2125,7 +2124,7 @@ static void motorP96WaitBack( pScanData ps )
 }
 
 /*.............................................................................
- * when loosing data, we use this function to go back some lines and read them
+ * when losing data, we use this function to go back some lines and read them
  * again...
  */
 static void motorP98WaitBack( pScanData ps )
@@ -2522,7 +2521,7 @@ static void motorP98SetSpeed( pScanData ps, Byte bSpeed, Bool fSetRunState )
 
 		if((lastFifoState > _P96_FIFOOVERFLOWTHRESH) &&
 													(bData < lastFifoState)) {
-			DBG( DBG_HIGH, "FIFO OVERFLOW, loosing data !!\n" );
+			DBG( DBG_HIGH, "FIFO OVERFLOW, losing data !!\n" );
 			overflow = _TRUE;
         }
         lastFifoState = bData;
