@@ -13,9 +13,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
    As a special exception, the authors of SANE give permission for
    additional uses of the libraries contained in this release of SANE.
@@ -419,7 +417,8 @@ umax_pp_auto_attach (SANEI_Config * config, SANE_Int safe)
  * device name to use for attach try.
  */
 static SANE_Status
-umax_pp_configure_attach (SANEI_Config * config, const char *devname)
+umax_pp_configure_attach (SANEI_Config * config, const char *devname,
+                          void __sane_unused__ *data)
 {
   const char *lp;
   SANE_Char *token;
@@ -961,7 +960,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 
   /* generic configure and attach function */
   status = sanei_configure_attach (UMAX_PP_CONFIG_FILE, &config,
-                                   umax_pp_configure_attach);
+                                   umax_pp_configure_attach, NULL);
 
   /* free option descriptors */
   for (i = 0; i < NUM_CFG_OPTIONS; i++)
@@ -2083,7 +2082,7 @@ sane_start (SANE_Handle handle)
                                 &(dev->th));
       /* we enlarged the scanning zone   */
       /* to allow reordering, we must    */
-      /* substract it from real scanning */
+      /* subtract it from real scanning */
       /* zone                            */
       dev->th -= points;
       DBG (64, "sane_start: bpp=%d,tw=%d,th=%d\n", dev->bpp, dev->tw,
